pro read_genxcat, t0x, t1x, outdata, topdir=topdir, genfiles=genfiles, $
		initialize=initialize, sublevel=sublevel, $
	        dbfiles=dbfiles, catfiles=catfiles
;
;+   Name: rd_genxcat
;
;    Purpose: read a catalog which uses 'genx' format files
;
;    History:
;      9-Mar-1998 - S.L.Freeland - long planned 'gen' function
;                   finally implemented for TRACE  
;
;    Restrictions:
;       catalog files have GMT of first record embedded in file name
;
;    Calls:
;       file2time, time2file, restgen, 
; 
;-
common read_genxcat, last_topdir, cfiles, cfids

initialize=keyword_set(initialize) or n_elements(cfiles) eq 0 or $
    n_elements(last_topdir) eq 0 

if initialize then begin
  if n_elements(sublevel) eq 0 then sublevel=0
  if data_chk(topdir,/string,/scalar) then begin
     if file_exist(topdir) then tdir=topdir else tdir=get_logenv(topdir)
  endif else tdir=curdir()
  for i=0,sublevel-1 do tdir=concat_dir(tdir,'*')
  catfiles=file_list(tdir,'*.genx',/cd)
  cfiles=catfiles
  cfids=extract_fid(cfiles)
  last_topdir=tdir
endif

fiddelim=(['.','_'])(strpos(cfids(0),'_') ne -1)

if n_elements(t0x) gt 0 then t0=t0x
if n_elements(t1x) gt 0 then t1=t1x

if n_elements(t0) gt 0 then begin
    fid0=time2file(t0,delim=fiddelim)
    if n_elements(t1) eq 0 then t1=timegrid(t0,/day,/string)
    fid1=time2file(t1,delim=fiddelim)
endif else begin
   fid0=cfids(0)
   fid1=cfids(n_elements(cfids)-1)
endelse  

ss=where(cfids ge fid0 and cfids lt fid1,sscnt)
if sscnt eq 0 then begin
  box_message,'No catalog entries satisfy your criteria...'
  return
endif  
if ss(0) gt 0 then ss=[ss(0)-1,ss]
sscnt=n_elements(ss)

restgen,file=catfiles(ss(0)),structs
outdata=temporary(structs)

for i=1, sscnt-1 do begin
   restgen,file=catfiles(ss(i)),structs
   outdata=[outdata,temporary(structs)]     
endfor

; trim times to actual if supplied
if n_elements(t1) gt 0 then begin
   ss=sel_timrange(outdata,t0,t1,/between)
   outdata=outdata(ss)
endif

return
end
